@@ -57,7 +57,6 @@
 #include "global.h"
 #include "exitcodes.h"
 #include "hash.h"
-#include "imap/imap_err.h"
 #include "mailbox.h"
 #include "mpool.h"
 #include "mboxlist.h"
@@ -66,6 +65,9 @@
 #include "xmalloc.h"
 #include "xstrlcpy.h"
 
+/* generated headers are not necessarily in current directory */
+#include "imap/imap_err.h"
+
 #define DB (config_seenstate_db)
 #define SUBDB (config_subscription_db)
 
@@ -103,7 +105,7 @@ static void run_users(void);
 static void make_report(const char *key, void *data, void *rock);
 static void process_seen(const char *path, const char *user);
 static void process_subs(const char *path, const char *user);
-static int do_mailbox(const char *name, int matchlen, int maycreate, void *rock);
+static int do_mailbox(struct findall_data *data, void *rock);
 
 int main(int argc,char **argv)
 {
@@ -116,72 +118,68 @@ int main(int argc,char **argv)
 
     strcpy(pattern, "*");
 
-    if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-	fatal("must run as the Cyrus user", EC_USAGE);
-    }
-
     report_end_time = now;
 
     while ((opt = getopt(argc, argv, "C:oud:D:p:l")) != EOF) {
-	switch (opt) {
-	case 'C': /* alt config file */
-	    alt_config = optarg;
-	    break;
+        switch (opt) {
+        case 'C': /* alt config file */
+            alt_config = optarg;
+            break;
 
-	case 'd':
-	    if (report_start_time != -1) usage();
-	    report_days = atoi(optarg);
-	    if (report_days <= 0) usage();
-	    break;
+        case 'd':
+            if (report_start_time != -1) usage();
+            report_days = atoi(optarg);
+            if (report_days <= 0) usage();
+            break;
 
-	case 'D': {
-	    unsigned  month = 0, day = 0, year = 0;
-	    struct tm date;
+        case 'D': {
+            unsigned  month = 0, day = 0, year = 0;
+            struct tm date;
 
-	    if (strlen(optarg) < 8 ||
-		sscanf(optarg, "%02u%02u%04u", &month, &day, &year) < 3) {
-		usage();
-	    }
-	    memset(&date, 0, sizeof(struct tm));
-	    date.tm_mon = month - 1;
-	    date.tm_mday = day;
-	    date.tm_year = year - 1900;
-	    report_start_time = mktime(&date);
+            if (strlen(optarg) < 8 ||
+                sscanf(optarg, "%02u%02u%04u", &month, &day, &year) < 3) {
+                usage();
+            }
+            memset(&date, 0, sizeof(struct tm));
+            date.tm_mon = month - 1;
+            date.tm_mday = day;
+            date.tm_year = year - 1900;
+            report_start_time = mktime(&date);
 
-	    if (optarg[8] == ':' && strlen(optarg+9) == 8) {
-		if (sscanf(optarg+9, "%02u%02u%04u", &month, &day, &year) < 3) {
-		    usage();
-		}
-		memset(&date, 0, sizeof(struct tm));
-		date.tm_mon = month - 1;
-		date.tm_mday = day;
-		date.tm_year = year - 1900;
-		report_end_time = mktime(&date);
-	    }
+            if (optarg[8] == ':' && strlen(optarg+9) == 8) {
+                if (sscanf(optarg+9, "%02u%02u%04u", &month, &day, &year) < 3) {
+                    usage();
+                }
+                memset(&date, 0, sizeof(struct tm));
+                date.tm_mon = month - 1;
+                date.tm_mday = day;
+                date.tm_year = year - 1900;
+                report_end_time = mktime(&date);
+            }
 
-	    break;
-	}
+            break;
+        }
 
-	case 'o':
-	    dosubs = 0;
-	    break;
+        case 'o':
+            dosubs = 0;
+            break;
 
-	case 'u':
-	    dousers = 1;
-	    break;
+        case 'u':
+            dousers = 1;
+            break;
 
-	case 'p':
-	    prune_months = atoi(optarg);
-	    if (prune_months <= 0) usage();
-	    break;
+        case 'p':
+            prune_months = atoi(optarg);
+            if (prune_months <= 0) usage();
+            break;
 
-	case 'l':
-	    long_report = dousers = 1;
-	    break;
+        case 'l':
+            long_report = dousers = 1;
+            break;
 
-	default:
-	    usage();
-	}
+        default:
+            usage();
+        }
     }
 
     /* Init Cyrus Backend Foo */
@@ -189,20 +187,20 @@ int main(int argc,char **argv)
 
     mboxlist_init(0);
     mboxlist_open(NULL);
-    
+
     /* Set namespace -- force standard (internal) */
     if ((r = mboxname_init_namespace(&arb_namespace, 1)) != 0) {
-	syslog(LOG_ERR, "%s", error_message(r));
-	fatal(error_message(r), EC_CONFIG);
+        syslog(LOG_ERR, "%s", error_message(r));
+        fatal(error_message(r), EC_CONFIG);
     }
 
     if (optind != argc) strlcpy(pattern, argv[optind], sizeof(pattern));
 
     if (report_start_time == -1) {
-	report_start_time = now - (report_days*60*60*24);
+        report_start_time = now - (report_days*60*60*24);
     }
     if (prune_months) {
-	prune_time = now - (prune_months*60*60*24*31);
+        prune_time = now - (prune_months*60*60*24*31);
     }
 
     /* Allocate our shared memory pools */
@@ -210,30 +208,27 @@ int main(int argc,char **argv)
     construct_hash_table(&mailbox_table, 2047, 1);
     construct_hash_table(&mboxname_table, 2047, 1);
 
-    /* Translate any separators in mailboxname */
-    mboxname_hiersep_tointernal(&arb_namespace, pattern, 0);
-    
     /* Get the mailbox list */
     fprintf(stderr, "Loading Mailboxes...");
-    (*arb_namespace.mboxlist_findall)(&arb_namespace, pattern, 1, 0, 0,
-				      do_mailbox, NULL);
+    mboxlist_findall(&arb_namespace, pattern, 1, 0, 0,
+                     do_mailbox, NULL);
 
     fprintf(stderr, "Done\nLoading Users");
-    
+
     /* Now do all the users */
     run_users();
 
-    fprintf(stderr, "Done\n");    
+    fprintf(stderr, "Done\n");
 
     /* And print the report */
-    hash_enumerate(&mboxname_table, make_report, NULL);    
+    hash_enumerate(&mboxname_table, make_report, NULL);
 
     /* Free Resources */
-    free_hash_table(&mailbox_table, NULL);    
+    free_hash_table(&mailbox_table, NULL);
     free_hash_table(&mboxname_table, NULL);
-    free_mpool(arb_pool);    
+    free_mpool(arb_pool);
     mboxlist_close();
-    mboxlist_done();    
+    mboxlist_done();
 
     cyrus_done();
 
@@ -243,25 +238,25 @@ int main(int argc,char **argv)
 static void usage(void)
 {
     fprintf(stderr,
-	    "usage: arbitron [-o] [-u] [-l] [-C alt_config] "
-	    "[-d days | -D mmddyyy[:mmddyyyy]]\n"
+            "usage: arbitron [-o] [-u] [-l] [-C alt_config] "
+            "[-d days | -D mmddyyy[:mmddyyyy]]\n"
             "                [-p months] [mboxpattern]\n");
     exit(EC_USAGE);
-}    
+}
 
-static int do_mailbox(const char *name, int matchlen __attribute__((unused)),
-		      int maycreate __attribute__((unused)),
-		      void *rock __attribute__((unused)))
+static int do_mailbox(struct findall_data *data, void *rock __attribute__((unused)))
 {
+    if (!data) return 0;
     int r;
     struct mailbox *mailbox = NULL;
+    const char *name = mbname_intname(data->mbname);
 
     r = mailbox_open_irl(name, &mailbox);
     if (r) return 0;
 
     struct arb_mailbox_data *d = mpool_malloc(arb_pool,
-					      sizeof(struct arb_mailbox_data));
-    
+                                              sizeof(struct arb_mailbox_data));
+
     d->nreaders = 0;
     d->nsubscribers = 0;
     d->readers = NULL;
@@ -278,94 +273,94 @@ static int do_mailbox(const char *name, int matchlen __attribute__((unused)),
 static void run_users(void)
 {
     char prefix[MAX_MAILBOX_PATH+1],path[MAX_MAILBOX_PATH+1],
-	file[MAX_MAILBOX_PATH+1];    
+        file[MAX_MAILBOX_PATH+1];
     DIR *dirp, *dirq;
     struct dirent *dirent1, *dirent2;
-    
+
     snprintf(prefix, sizeof(prefix), "%s%s", config_dir, FNAME_USERDIR);
-    
+
     dirp = opendir(prefix);
     if(!dirp) {
-	fatal("can't open user directory", EC_SOFTWARE);
+        fatal("can't open user directory", EC_SOFTWARE);
     }
 
     while((dirent1 = readdir(dirp)) != NULL) {
-	if(!strcmp(dirent1->d_name, ".") || !strcmp(dirent1->d_name,"..")) {
-	    continue;	    
-	}
-	
-	snprintf(path, sizeof(path), "%s%s", prefix, dirent1->d_name);
-/*	printf("trying %s\n",path); */
-	
-	dirq = opendir(path);
-	if(dirq) {	    
-	    fprintf(stderr, ".");	    
-	    while(dirq && ((dirent2 = readdir(dirq)) != NULL)) {
-		size_t len;
-		
-		if(!strcmp(dirent2->d_name, ".") ||
-		   !strcmp(dirent2->d_name,"..")) {
-		    continue;	    
-		}
+        if(!strcmp(dirent1->d_name, ".") || !strcmp(dirent1->d_name,"..")) {
+            continue;
+        }
 
-		len = strlen(dirent2->d_name);
+        snprintf(path, sizeof(path), "%s%s", prefix, dirent1->d_name);
+/*      printf("trying %s\n",path); */
 
-	        /* 5 is magic number for strlen(".seen") and
-		   4 is the magic number for strlen(".sub") */
-		if(len > 4) {
-		    char *user = NULL;
+        dirq = opendir(path);
+        if(dirq) {
+            fprintf(stderr, ".");
+            while(dirq && ((dirent2 = readdir(dirq)) != NULL)) {
+                size_t len;
 
-		    snprintf(file, sizeof(file),
-			     "%s/%s", path, dirent2->d_name);
-/*		    printf("got file %s\n",file); */
-		    if(len > 5 &&
-		       !strcmp(dirent2->d_name + len - 5, ".seen")) {
-			if (dousers) {
-			    user = mpool_strndup(arb_pool, dirent2->d_name,
-						 len-5);
-			}
-			process_seen(file, user);
-		    } else if (dosubs &&
-			       !strcmp(dirent2->d_name + len - 4, ".sub")) {
-			if (dousers) {
-			    user = mpool_strndup(arb_pool, dirent2->d_name,
-						 len-4);
-			}
-			process_subs(file, user);
-		    }
-		}
-	    }
-	    closedir(dirq);
-	}
-	    
-    }    
+                if(!strcmp(dirent2->d_name, ".") ||
+                   !strcmp(dirent2->d_name,"..")) {
+                    continue;
+                }
+
+                len = strlen(dirent2->d_name);
+
+                /* 5 is magic number for strlen(".seen") and
+                   4 is the magic number for strlen(".sub") */
+                if(len > 4) {
+                    char *user = NULL;
+
+                    snprintf(file, sizeof(file),
+                             "%s/%s", path, dirent2->d_name);
+/*                  printf("got file %s\n",file); */
+                    if(len > 5 &&
+                       !strcmp(dirent2->d_name + len - 5, ".seen")) {
+                        if (dousers) {
+                            user = mpool_strndup(arb_pool, dirent2->d_name,
+                                                 len-5);
+                        }
+                        process_seen(file, user);
+                    } else if (dosubs &&
+                               !strcmp(dirent2->d_name + len - 4, ".sub")) {
+                        if (dousers) {
+                            user = mpool_strndup(arb_pool, dirent2->d_name,
+                                                 len-4);
+                        }
+                        process_subs(file, user);
+                    }
+                }
+            }
+            closedir(dirq);
+        }
+
+    }
     closedir(dirp);
 
 }
 
 static int process_user_cb(void *rockp,
-			   const char *key, size_t keylen,
-			   const char *tmpdata __attribute__((unused)),
-			   size_t tmpdatalen __attribute__((unused))) 
+                           const char *key, size_t keylen,
+                           const char *tmpdata __attribute__((unused)),
+                           size_t tmpdatalen __attribute__((unused)))
 {
     /* Only called to do deletes */
 /*    printf("pruning entry\n"); */
-    
-    cyrusdb_delete((struct db *)rockp, key, keylen, NULL, 0);    
 
-    return 0;    
+    cyrusdb_delete((struct db *)rockp, key, keylen, NULL, 0);
+
+    return 0;
 }
 
 /* We can cheat and do all we need to in this function */
 static int process_user_p(void *rockp,
-			  const char *key,
-			  size_t keylen,
-			  const char *data,
-			  size_t datalen __attribute__((unused))) 
+                          const char *key,
+                          size_t keylen,
+                          const char *data,
+                          size_t datalen __attribute__((unused)))
 {
-    int ret = 0;    
+    int ret = 0;
     long version, lastread;
-    char *p;    
+    char *p;
     char buf[64];
     struct arb_mailbox_data *mbox;
     const char *user = (const char *) rockp;
@@ -375,7 +370,7 @@ static int process_user_p(void *rockp,
     if (version < 0) abort();
     /* xxx not checking version */
     lastread = strtol(data, &p, 10); data = p;
-    
+
     memcpy(buf, key, keylen);
     buf[keylen] = '\0';
 
@@ -383,55 +378,55 @@ static int process_user_p(void *rockp,
 
     if(mbox && lastread >= report_start_time &&
        lastread <= report_end_time) {
-/*	printf("got %s\n", mbox->name);	     */
-	mbox->nreaders++;
-	if (user) {
-	    struct user_list *u = mpool_malloc(arb_pool,
-					       sizeof(struct user_list));
+/*      printf("got %s\n", mbox->name);      */
+        mbox->nreaders++;
+        if (user) {
+            struct user_list *u = mpool_malloc(arb_pool,
+                                               sizeof(struct user_list));
 
-	    u->user = user;
-	    u->next = mbox->readers;
-	    mbox->readers = u;
-	}
+            u->user = user;
+            u->next = mbox->readers;
+            mbox->readers = u;
+        }
     }
 
     /* Check for pruning even if mailbox isn't valid */
     if(lastread < prune_time) {
-	ret = 1;
-    }	
+        ret = 1;
+    }
 
     /* Only return true if we need to prune this guy */
-    return ret;    
+    return ret;
 }
 
 static void process_seen(const char *path, const char *user)
 {
-    int r;    
+    int r;
     struct db *tmp = NULL;
 
     r = cyrusdb_open(DB, path, 0, &tmp);
     if(r) goto done;
-    
+
     cyrusdb_foreach(tmp, "", 0, process_user_p, process_user_cb,
-		(void *) user, NULL);
+                (void *) user, NULL);
 
  done:
     if(tmp) cyrusdb_close(tmp);
 }
 
 static int process_subs_cb(void *rockp __attribute__((unused)),
-			   const char *key __attribute__((unused)),
-			   size_t keylen __attribute__((unused)),
-			   const char *tmpdata __attribute__((unused)),
-			   size_t tmpdatalen __attribute__((unused))) 
+                           const char *key __attribute__((unused)),
+                           size_t keylen __attribute__((unused)),
+                           const char *tmpdata __attribute__((unused)),
+                           size_t tmpdatalen __attribute__((unused)))
 {
     return 0;
 }
 
 static int process_subs_p(void *rockp,
-			  const char *key, size_t keylen,
-			  const char *tmpdata __attribute__((unused)),
-			  size_t tmpdatalen __attribute__((unused))) 
+                          const char *key, size_t keylen,
+                          const char *tmpdata __attribute__((unused)),
+                          size_t tmpdatalen __attribute__((unused)))
 {
     struct arb_mailbox_data *mbox;
     char buf[MAX_MAILBOX_BUFFER];
@@ -445,16 +440,16 @@ static int process_subs_p(void *rockp,
     mbox = hash_lookup(buf, &mboxname_table);
 
     if(mbox) {
-/*	printf("got sub %s\n", buf); */
-	mbox->nsubscribers++;
-	if (user) {
-	    struct user_list *u = mpool_malloc(arb_pool,
-					       sizeof(struct user_list));
+/*      printf("got sub %s\n", buf); */
+        mbox->nsubscribers++;
+        if (user) {
+            struct user_list *u = mpool_malloc(arb_pool,
+                                               sizeof(struct user_list));
 
-	    u->user = user;
-	    u->next = mbox->subscribers;
-	    mbox->subscribers = u;
-	}
+            u->user = user;
+            u->next = mbox->subscribers;
+            mbox->subscribers = u;
+        }
     }
 
     return 0; /* never do callback */
@@ -462,14 +457,14 @@ static int process_subs_p(void *rockp,
 
 static void process_subs(const char *path, const char *user)
 {
-    int r;    
+    int r;
     struct db *tmp = NULL;
 
     r = cyrusdb_open(SUBDB, path, 0, &tmp);
     if(r) goto done;
-    
+
     cyrusdb_foreach(tmp, "", 0, process_subs_p, process_subs_cb,
-		   (void *) user, NULL);
+                   (void *) user, NULL);
 
  done:
     if(tmp) cyrusdb_close(tmp);
@@ -480,9 +475,9 @@ static void report_users(struct user_list *u)
     char sep = ':';
 
     while (u) {
-	printf("%c%s", sep, u->user);
-	sep = ',';
-	u = u->next;
+        printf("%c%s", sep, u->user);
+        sep = ',';
+        u = u->next;
     }
 }
 
@@ -492,43 +487,40 @@ static void long_report_users(struct user_list *u, const char *mbox, char type)
     struct tm *tm;
 
     while (u) {
-	printf("%s,%s,%c,", mbox, u->user, type);
-	tm = localtime(&report_start_time);
-	strftime(buf, sizeof(buf), "%m-%d-%Y %H:%M:%S", tm);
-	printf("%s,", buf);
+        printf("%s,%s,%c,", mbox, u->user, type);
+        tm = localtime(&report_start_time);
+        strftime(buf, sizeof(buf), "%m-%d-%Y %H:%M:%S", tm);
+        printf("%s,", buf);
 
-	tm = localtime(&report_end_time);
-	strftime(buf, sizeof(buf), "%m-%d-%Y %H:%M:%S", tm);
-	printf("%s\n", buf);
-	u = u->next;
+        tm = localtime(&report_end_time);
+        strftime(buf, sizeof(buf), "%m-%d-%Y %H:%M:%S", tm);
+        printf("%s\n", buf);
+        u = u->next;
     }
 }
 
 static void make_report(const char *key, void *data, void *rock __attribute__((unused)))
 {
     struct arb_mailbox_data *mbox = (struct arb_mailbox_data *)data;
-    char *extkey;
 
     /* Skip underread user mailboxes */
     if(!strncasecmp(key, "user.", 5) && mbox->nreaders <= 1)
-	return;    
+        return;
 
-    extkey = xstrdup(key);
-    mboxname_hiersep_toexternal(&arb_namespace, extkey, 0);
-    key = extkey;
+    char *extname = mboxname_to_external(key, &arb_namespace, NULL);
 
     if (long_report) {
-	long_report_users(mbox->readers, key, 'r');
-	long_report_users(mbox->subscribers, key, 's');
+        long_report_users(mbox->readers, extname, 'r');
+        long_report_users(mbox->subscribers, extname, 's');
     }
     else {
-	printf("%s %d", key, mbox->nreaders);
-	if (dousers) report_users(mbox->readers);
-	if (dosubs) {
-	    printf(" %d", mbox->nsubscribers);
-	if (dousers) report_users(mbox->subscribers);
-	}
-	printf("\n");
+        printf("%s %d", extname, mbox->nreaders);
+        if (dousers) report_users(mbox->readers);
+        if (dosubs) {
+            printf(" %d", mbox->nsubscribers);
+        if (dousers) report_users(mbox->subscribers);
+        }
+        printf("\n");
     }
-    free(extkey);
+    free(extname);
 }
