@@ -98,10 +98,6 @@ int main(int argc, char **argv)
     time_t runattime = 0;
     int upgrade = 0;
 
-    if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-        fatal("must run as the Cyrus user", EC_USAGE);
-    }
-
     while ((opt = getopt(argc, argv, "C:dt:U")) != EOF) {
         switch (opt) {
         case 'C': /* alt config file */
