@@ -82,10 +82,6 @@ int main(int argc, char *argv[])
     char *alt_config = NULL;
     struct incremental_record irec;
 
-    if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-        fatal("must run as the Cyrus user", EC_USAGE);
-    }
-
     while ((option = getopt(argc, argv, "vC:")) != EOF) {
         switch (option) {
         case 'v':
