@@ -266,10 +266,6 @@ int main(int argc, char *argv[])
     int newnews = 1;
     char *datefmt = "%y%m%d %H%M%S";
 
-    if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-        fatal("must run as the Cyrus user", EC_USAGE);
-    }
-
     while ((opt = getopt(argc, argv, "C:s:w:f:a:p:ny")) != EOF) {
         switch (opt) {
         case 'C': /* alt config file */
