@@ -252,9 +252,12 @@ static int meth_get(struct transaction_t *txn,
                 resp_body->lastmod = lastmod;
                 resp_body->maxage = 31536000;  /* 1 year */
                 txn->flags.cc |= CC_MAXAGE;
+                if (httpd_userid) txn->flags.cc |= CC_PRIVATE;
 
                 if (precond != HTTP_NOT_MODIFIED) break;
 
+                GCC_FALLTHROUGH
+
             default:
                 /* We failed a precondition - don't perform the request */
                 ret = precond;
@@ -630,6 +633,8 @@ static int list_feeds(struct transaction_t *txn)
 
         if (precond != HTTP_NOT_MODIFIED) break;
 
+        GCC_FALLTHROUGH
+
     default:
         /* We failed a precondition - don't perform the request */
         ret = precond;
@@ -818,6 +823,8 @@ static int list_messages(struct transaction_t *txn, struct mailbox *mailbox)
 
         if (precond != HTTP_NOT_MODIFIED) break;
 
+        GCC_FALLTHROUGH
+
     default:
         /* We failed a precondition - don't perform the request */
         return precond;
