@@ -416,7 +416,7 @@ EXPORTED void eatline(struct protstream *pin, int c)
         /* Several of the parser helper functions return EOF
            even if an unexpected character (other than EOF) is received. 
            We need to confirm that the stream is actually at EOF. */
-        if (c == EOF && prot_IS_EOF(pin)) return;
+        if (c == EOF && (prot_IS_EOF(pin) || prot_IS_ERROR(pin))) return;
 
         /* see if it's a literal */
         if (c == '{') {
@@ -729,11 +729,11 @@ static void date_range(search_expr_t *parent, const char *aname,
 
     e = search_expr_new(parent, SEOP_LT);
     e->attr = attr;
-    e->value.u = end;
+    e->value.t = end;
 
     e = search_expr_new(parent, SEOP_GE);
     e->attr = attr;
-    e->value.u = start;
+    e->value.t = start;
 }
 
 /*
@@ -749,7 +749,7 @@ static int get_search_criterion(struct protstream *pin,
     int c;
     int keep_charset = 0;
     time_t start, end, now = time(0);
-    uint32_t u;
+    int32_t i32;
     int hasconv = config_getswitch(IMAPOPT_CONVERSATIONS);
 
     if (base->state & GETSEARCH_CHARSET_FIRST) {
@@ -823,7 +823,7 @@ static int get_search_criterion(struct protstream *pin,
             if (c == EOF) goto baddate;
             e = search_expr_new(parent, SEOP_LT);
             e->attr = search_attr_find("internaldate");
-            e->value.u = start;
+            e->value.t = start;
         }
         else if (!strcmp(criteria.s, "bcc")) {      /* RFC 3501 */
             if (c != ' ') goto missingarg;
@@ -977,11 +977,11 @@ static int get_search_criterion(struct protstream *pin,
     case 'l':
         if (!strcmp(criteria.s, "larger")) {            /* RFC 3501 */
             if (c != ' ') goto missingarg;
-            c = getint32(pin, (int32_t *)&u);
+            c = getint32(pin, &i32);
             if (c == EOF) goto badnumber;
             e = search_expr_new(parent, SEOP_GT);
             e->attr = search_attr_find("size");
-            e->value.u = u;
+            e->value.u = i32;
         }
         else goto badcri;
         break;
@@ -1036,11 +1036,11 @@ static int get_search_criterion(struct protstream *pin,
         }
         else if (!strcmp(criteria.s, "older")) {    /* RFC 5032 */
             if (c != ' ') goto missingarg;
-            c = getint32(pin, (int32_t *)&u);
+            c = getint32(pin, &i32);
             if (c == EOF) goto badinterval;
             e = search_expr_new(parent, SEOP_LE);
             e->attr = search_attr_find("internaldate");
-            e->value.u = now - u;
+            e->value.t = now - i32;
         }
         else if (!strcmp(criteria.s, "on")) {   /* RFC 3501 */
             if (c != ' ') goto missingarg;
@@ -1074,7 +1074,7 @@ static int get_search_criterion(struct protstream *pin,
             if (c == EOF) goto baddate;
             e = search_expr_new(parent, SEOP_LT);
             e->attr = search_attr_find("sentdate");
-            e->value.u = start;
+            e->value.t = start;
         }
         else if (!strcmp(criteria.s, "senton")) {       /* RFC 3501 */
             if (c != ' ') goto missingarg;
@@ -1088,7 +1088,7 @@ static int get_search_criterion(struct protstream *pin,
             if (c == EOF) goto baddate;
             e = search_expr_new(parent, SEOP_GE);
             e->attr = search_attr_find("sentdate");
-            e->value.u = start;
+            e->value.t = start;
         }
         else if (!strcmp(criteria.s, "since")) {    /* RFC 3501 */
             if (c != ' ') goto missingarg;
@@ -1096,15 +1096,15 @@ static int get_search_criterion(struct protstream *pin,
             if (c == EOF) goto baddate;
             e = search_expr_new(parent, SEOP_GE);
             e->attr = search_attr_find("internaldate");
-            e->value.u = start;
+            e->value.t = start;
         }
         else if (!strcmp(criteria.s, "smaller")) {  /* RFC 3501 */
             if (c != ' ') goto missingarg;
-            c = getint32(pin, (int32_t *)&u);
+            c = getint32(pin, &i32);
             if (c == EOF) goto badnumber;
             e = search_expr_new(parent, SEOP_LT);
             e->attr = search_attr_find("size");
-            e->value.u = u;
+            e->value.u = i32;
         }
         else if (!strcmp(criteria.s, "spamabove")) {  /* nonstandard */
             if (c != ' ') goto missingarg;
@@ -1212,11 +1212,11 @@ static int get_search_criterion(struct protstream *pin,
     case 'y':
         if (!strcmp(criteria.s, "younger")) {           /* RFC 5032 */
             if (c != ' ') goto missingarg;
-            c = getint32(pin, (int32_t *)&u);
+            c = getint32(pin, &i32);
             if (c == EOF) goto badinterval;
             e = search_expr_new(parent, SEOP_GE);
             e->attr = search_attr_find("internaldate");
-            e->value.u = now - u;
+            e->value.t = now - i32;
         }
         else goto badcri;
         break;
