@@ -85,10 +85,6 @@ main(int argc, char **argv)
   int opt;              /* getopt() returns an int */
   char *alt_config = NULL;
 
-  if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-      fatal("must run as the Cyrus user", EC_USAGE);
-  }
-
   while ((opt = getopt(argc, argv, "C:qsm")) != EOF) {
     switch(opt) {
     case 'C': /* alt config file */
