@@ -130,10 +130,6 @@ int main(int argc,char **argv)
     int do_report = 1;
     char *alt_config = NULL, *domain = NULL;
 
-    if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-        fatal("must run as the Cyrus user", EC_USAGE);
-    }
-
     while ((opt = getopt(argc, argv, "C:d:fqJZ")) != EOF) {
         switch (opt) {
         case 'C': /* alt config file */
@@ -408,11 +404,12 @@ int buildquotalist(char *domain, char **roots, int nroots)
 
 static int findroot(const char *name, int *thisquota)
 {
-    int i;
+    int i = config_getswitch(IMAPOPT_IMPROVED_MBOXLIST_SORT)
+            ? quota_todo : 0;
 
     *thisquota = -1;
 
-    for (i = quota_todo; i < quota_num; i++) {
+    for (; i < quota_num; i++) {
         const char *root = quotaroots[i].name;
 
         /* have we already passed the name, then there can
@@ -446,21 +443,23 @@ static int fixquota_dombox(const mbentry_t *mbentry, void *rock)
     int thisquota = -1;
     struct txn *txn = NULL;
 
-    while (quota_todo < quota_num) {
-        const char *root = quotaroots[quota_todo].name;
+    if (config_getswitch(IMAPOPT_IMPROVED_MBOXLIST_SORT)) {
+        while (quota_todo < quota_num) {
+            const char *root = quotaroots[quota_todo].name;
 
-        /* in the future, definitely don't close yet */
-        if (compar(mbentry->name, root) < 0)
-            break;
+            /* in the future, definitely don't close yet */
+            if (compar(mbentry->name, root) < 0)
+                break;
 
-        /* inside the first root, don't close yet */
-        if (mboxname_is_prefix(mbentry->name, root))
-            break;
+            /* inside the first root, don't close yet */
+            if (mboxname_is_prefix(mbentry->name, root))
+                break;
 
-        /* finished, close out now */
-        r = fixquota_finish(quota_todo);
-        quota_todo++;
-        if (r) goto done;
+            /* finished, close out now */
+            r = fixquota_finish(quota_todo);
+            quota_todo++;
+            if (r) goto done;
+        }
     }
 
     test_sync_wait(mbentry->name);
