@@ -472,10 +472,7 @@ EXPORTED int quota_update_useds(const char *quotaroot,
     if (!r) {
         int res;
         int cmp = 1;
-        if (q.scanmbox) {
-            /* XXX this will crash if mboxname was null...
-             * https://github.com/cyrusimap/cyrus-imapd/issues/2808
-             */
+        if (mboxname && q.scanmbox) {
             cmp = cyrusdb_compar(qdb, mboxname, strlen(mboxname),
                                  q.scanmbox, strlen(q.scanmbox));
         }
