@@ -739,13 +739,13 @@ static const char *make_cyrusid(const struct message_guid *guid)
 /* XXX - replace with cyrus_mkdir and cyrus_copyfile */
 static void remove_dir(const char *dir)
 {
-    run_command("/bin/rm", "-rf", dir, (char *)NULL);
+    run_command(RM_BIN, "-rf", dir, (char *)NULL);
 }
 
 static int copy_files(const char *fromdir, const char *todir)
 {
     char *fromdir2 = strconcat(fromdir, "/", (char *)NULL);
-    int r = run_command("/usr/bin/rsync", "-a", fromdir2, todir, (char *)NULL);
+    int r = run_command(RSYNC_BIN, "-a", fromdir2, todir, (char *)NULL);
 
     free(fromdir2);
     return r;
@@ -1756,7 +1756,7 @@ static int end_message_snippets(search_text_receiver_t *rx)
             generate_snippet_terms(tr->snipgen, seg->part, tr->root);
         }
 
-        r = xapian_snipgen_doc_part(tr->snipgen, &seg->text);
+        r = xapian_snipgen_doc_part(tr->snipgen, &seg->text, seg->part);
         if (r) break;
 
         last_part = seg->part;
