@@ -51,15 +51,15 @@ typedef hash_table *hdrcache_t;
 
 hdrcache_t spool_new_hdrcache(void);
 void spool_cache_header(char *name, char *body, hdrcache_t cache);
-void spool_replace_header(char *name, char *newvalue,
-			  hdrcache_t cache);
+void spool_replace_header(char *name, char *newvalue, hdrcache_t cache);
+void spool_remove_header(char *name, hdrcache_t cache);
 int spool_fill_hdrcache(struct protstream *fin, FILE *fout, hdrcache_t cache,
-			const char **skipheaders);
+                        const char **skipheaders);
 const char **spool_getheader(hdrcache_t cache, const char *phead);
 void spool_free_hdrcache(hdrcache_t cache);
 void spool_enum_hdrcache(hdrcache_t cache,
-			 void (*proc)(const char *, const char *, void *),
-			 void *rock);
+                         void (*proc)(const char *, const char *, void *),
+                         void *rock);
 int spool_copy_msg(struct protstream *fin, FILE *fout);
 
 #endif
