@@ -185,10 +185,6 @@ main(int argc, char **argv)
     int force = 0;
     int i;
 
-    if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-        fatal("must run as the Cyrus user", EC_USAGE);
-    }
-
     setbuf(stdout, NULL);
 
     while ((opt = getopt(argc, argv, "C:vfL")) != EOF) {
