@@ -227,7 +227,7 @@ static RSA *tmp_rsa_cb(SSL * s __attribute__((unused)),
 }
 #endif
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || (defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER < 0x20700000L)
 /* replacements for new 1.1 API accessors */
 /* XXX probably put these somewhere central */
 static int DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g)
@@ -738,6 +738,15 @@ EXPORTED int     tls_init_serverengine(const char *ident,
 
     const char *tls_versions = config_getstring(IMAPOPT_TLS_VERSIONS);
 
+    if (strstr(tls_versions, "tls1_3") == NULL) {
+#if (OPENSSL_VERSION_NUMBER >= 0x1010100fL)
+        //syslog(LOG_DEBUG, "TLS server engine: Disabled TLSv1.3");
+        off |= SSL_OP_NO_TLSv1_3;
+#else
+        syslog(LOG_ERR, "ERROR: TLSv1.3 configured, OpenSSL < 1.1.1 insufficient");
+#endif // (OPENSSL_VERSION_NUMBER >= 0x1010100fL)
+    }
+
     if (strstr(tls_versions, "tls1_2") == NULL) {
 #if (OPENSSL_VERSION_NUMBER >= 0x1000105fL)
         //syslog(LOG_DEBUG, "TLS server engine: Disabled TLSv1.2");
