@@ -63,10 +63,6 @@ int main(int argc, char *argv[])
     int opt,r;
     char *alt_config = NULL;
 
-    if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
-        fatal("must run as the Cyrus user", EC_USAGE);
-    }
-
     while ((opt = getopt(argc, argv, "C:")) != EOF) {
         switch (opt) {
         case 'C': /* alt config file */
