@@ -38,8 +38,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: gmtoff_tm.c,v 1.12 2008/03/24 17:43:09 murch Exp $
  */
 
 #include <config.h>
@@ -47,9 +45,7 @@
 /*
  * Returns the GMT offset of the struct tm 'tm', obtained from 'time'.
  */
-int gmtoff_of(tm, time)
-struct tm *tm;
-time_t time __attribute__((unused));
+EXPORTED int gmtoff_of(struct tm *tm, time_t time __attribute__((unused)))
 {
     return tm->tm_gmtoff;
 }
