@@ -189,7 +189,12 @@ EXPORTED const char *config_partitiondir(const char *partition)
     if (strlcat(buf, partition, sizeof(buf)) >= sizeof(buf))
         return 0;
 
-    return config_getoverflowstring(buf, NULL);
+    const char *dir = config_getoverflowstring(buf, NULL);
+    if (!dir)
+        syslog(LOG_WARNING, "requested partition directory for unknown partition '%s'",
+                            partition);
+
+    return dir;
 }
 
 EXPORTED const char *config_metapartitiondir(const char *partition)
@@ -201,7 +206,12 @@ EXPORTED const char *config_metapartitiondir(const char *partition)
     if (strlcat(buf, partition, sizeof(buf)) >= sizeof(buf))
         return 0;
 
-    return config_getoverflowstring(buf, NULL);
+    const char *dir = config_getoverflowstring(buf, NULL);
+    if (!dir)
+        syslog(LOG_DEBUG, "requested meta partition directory for unknown partition '%s'",
+                          partition);
+
+    return dir;
 }
 
 EXPORTED const char *config_archivepartitiondir(const char *partition)
@@ -213,7 +223,12 @@ EXPORTED const char *config_archivepartitiondir(const char *partition)
     if(strlcat(buf, partition, sizeof(buf)) >= sizeof(buf))
         return 0;
 
-    return config_getoverflowstring(buf, NULL);
+    const char *dir = config_getoverflowstring(buf, NULL);
+    if (!dir)
+        syslog(LOG_DEBUG, "requested archive partition directory for unknown partition '%s'",
+                          partition);
+
+    return dir;
 }
 
 EXPORTED const char *config_backupstagingpath(void)
@@ -672,7 +687,7 @@ static void config_read_file(const char *filename)
                 ) {
 
                 sprintf(errbuf,
-                        "option '%s' was specified twice in config file (second occurance on line %d)",
+                        "option '%s' was specified twice in config file (second occurrence on line %d)",
                         fullkey, lineno);
                 fatal(errbuf, EC_CONFIG);
 
@@ -816,7 +831,7 @@ static void config_read_file(const char *filename)
             val = hash_insert(key, newval, &confighash);
             if (val != newval) {
                 snprintf(errbuf, sizeof(errbuf),
-                        "option '%s' was specified twice in config file (second occurance on line %d)",
+                        "option '%s' was specified twice in config file (second occurrence on line %d)",
                         fullkey, lineno);
                 fatal(errbuf, EC_CONFIG);
             }
