@@ -58,10 +58,10 @@ EXPORTED void strarray_fini(strarray_t *sa)
     int i;
 
     if (!sa)
-	return;
+        return;
     for (i = 0 ; i < sa->count ; i++) {
-	free(sa->data[i]);
-	sa->data[i] = NULL;
+        free(sa->data[i]);
+        sa->data[i] = NULL;
     }
     free(sa->data);
     sa->data = NULL;
@@ -72,7 +72,7 @@ EXPORTED void strarray_fini(strarray_t *sa)
 EXPORTED void strarray_free(strarray_t *sa)
 {
     if (!sa)
-	return;
+        return;
     strarray_fini(sa);
     free(sa);
 }
@@ -84,11 +84,11 @@ EXPORTED void strarray_free(strarray_t *sa)
  * index, so that we can pass data[] to execve() or other routines that
  * assume a NULL terminator.
  */
-#define QUANTUM	    16
+#define QUANTUM     16
 static void ensure_alloc(strarray_t *sa, int newalloc)
 {
     if (newalloc < sa->alloc)
-	return;
+        return;
     newalloc = ((newalloc + QUANTUM) / QUANTUM) * QUANTUM;
     sa->data = xrealloc(sa->data, sizeof(char *) * newalloc);
     memset(sa->data+sa->alloc, 0, sizeof(char *) * (newalloc-sa->alloc));
@@ -109,9 +109,9 @@ static void ensure_alloc(strarray_t *sa, int newalloc)
 static inline int adjust_index_ro(const strarray_t *sa, int idx)
 {
     if (idx >= sa->count)
-	return -1;
+        return -1;
     else if (idx < 0)
-	idx += sa->count;
+        idx += sa->count;
     return idx;
 }
 
@@ -124,17 +124,17 @@ static inline int adjust_index_ro(const strarray_t *sa, int idx)
 static inline int adjust_index_rw(strarray_t *sa, int idx, int grow)
 {
     if (idx >= sa->count) {
-	/* expanding the array as a side effect @idx pointing
-	 * outside the current bounds, plus perhaps @grow */
-	ensure_alloc(sa, idx+grow);
+        /* expanding the array as a side effect @idx pointing
+         * outside the current bounds, plus perhaps @grow */
+        ensure_alloc(sa, idx+grow);
     } else if (idx < 0) {
-	/* adjust Perl-style negative indeces */
-	idx += sa->count;
-	if (idx >= 0 && grow)
-	    ensure_alloc(sa, sa->count+grow);
+        /* adjust Perl-style negative indeces */
+        idx += sa->count;
+        if (idx >= 0 && grow)
+            ensure_alloc(sa, sa->count+grow);
     } else if (grow) {
-	/* expanding the array due to an insert or append */
-	ensure_alloc(sa, sa->count+grow);
+        /* expanding the array due to an insert or append */
+        ensure_alloc(sa, sa->count+grow);
     }
     return idx;
 }
@@ -144,9 +144,11 @@ EXPORTED strarray_t *strarray_dup(const strarray_t *sa)
     strarray_t *new = strarray_new();
     int i;
 
+    if (!sa) return new;
+
     strarray_truncate(new, sa->count);
     for (i = 0 ; i < sa->count ; i++)
-	new->data[i] = xstrdup(sa->data[i]);
+        new->data[i] = xstrdup(sa->data[i]);
     return new;
 }
 
@@ -155,6 +157,13 @@ EXPORTED int strarray_append(strarray_t *sa, const char *s)
     return strarray_appendm(sa, xstrdup(s));
 }
 
+EXPORTED void strarray_cat(strarray_t *dest, const strarray_t *src)
+{
+    int i;
+    for (i = 0 ; i < src->count ; i++)
+        strarray_append(dest, strarray_nth(src, i));
+}
+
 EXPORTED int strarray_add(strarray_t *sa, const char *s)
 {
     int pos = strarray_find(sa, s, 0);
@@ -184,28 +193,28 @@ static void _strarray_set(strarray_t *sa, int idx, char *s)
     sa->data[idx] = s;
     /* adjust the count if we just sparsely expanded the array */
     if (s && idx >= sa->count)
-	sa->count = idx+1;
+        sa->count = idx+1;
 }
 
 EXPORTED void strarray_set(strarray_t *sa, int idx, const char *s)
 {
     if ((idx = adjust_index_rw(sa, idx, 0)) < 0)
-	return;
+        return;
     _strarray_set(sa, idx, xstrdupnull(s));
 }
 
 EXPORTED void strarray_setm(strarray_t *sa, int idx, char *s)
 {
     if ((idx = adjust_index_rw(sa, idx, 0)) < 0)
-	return;
+        return;
     _strarray_set(sa, idx, s);
 }
 
 static inline void _strarray_insert(strarray_t *sa, int idx, char *s)
 {
     if (idx < sa->count)
-	memmove(sa->data+idx+1, sa->data+idx,
-		sizeof(char *) * (sa->count-idx));
+        memmove(sa->data+idx+1, sa->data+idx,
+                sizeof(char *) * (sa->count-idx));
     sa->data[idx] = s;
     sa->count++;
 }
@@ -213,14 +222,14 @@ static inline void _strarray_insert(strarray_t *sa, int idx, char *s)
 EXPORTED void strarray_insert(strarray_t *sa, int idx, const char *s)
 {
     if ((idx = adjust_index_rw(sa, idx, 1)) < 0)
-	return;
+        return;
     _strarray_insert(sa, idx, xstrdup(s));
 }
 
 EXPORTED void strarray_insertm(strarray_t *sa, int idx, char *s)
 {
     if ((idx = adjust_index_rw(sa, idx, 1)) < 0)
-	return;
+        return;
     _strarray_insert(sa, idx, s);
 }
 
@@ -228,12 +237,12 @@ EXPORTED char *strarray_remove(strarray_t *sa, int idx)
 {
     char *s;
     if ((idx = adjust_index_ro(sa, idx)) < 0)
-	return NULL;
+        return NULL;
     s = sa->data[idx];
     sa->count--;
     if (idx < sa->count)
-	memmove(sa->data+idx, sa->data+idx+1,
-		sizeof(char *) * (sa->count-idx));
+        memmove(sa->data+idx, sa->data+idx+1,
+                sizeof(char *) * (sa->count-idx));
     return s;
 }
 
@@ -242,10 +251,10 @@ EXPORTED void strarray_remove_all(strarray_t *sa, const char *s)
     int i = 0;
 
     for (;;) {
-	i = strarray_find(sa, s, i);
-	if (i < 0)
-	    break;
-	free(strarray_remove(sa, i));
+        i = strarray_find(sa, s, i);
+        if (i < 0)
+            break;
+        free(strarray_remove(sa, i));
     }
 }
 
@@ -254,10 +263,10 @@ EXPORTED void strarray_remove_all_case(strarray_t *sa, const char *s)
     int i = 0;
 
     for (;;) {
-	i = strarray_find_case(sa, s, i);
-	if (i < 0)
-	    break;
-	free(strarray_remove(sa, i));
+        i = strarray_find_case(sa, s, i);
+        if (i < 0)
+            break;
+        free(strarray_remove(sa, i));
     }
 }
 
@@ -266,15 +275,15 @@ EXPORTED void strarray_truncate(strarray_t *sa, int newlen)
     int i;
 
     if (newlen == sa->count)
-	return;
+        return;
 
     if (newlen > sa->count) {
-	ensure_alloc(sa, newlen);
+        ensure_alloc(sa, newlen);
     } else {
-	for (i = newlen ; i < sa->count ; i++) {
-	    free(sa->data[i]);
-	    sa->data[i] = NULL;
-	}
+        for (i = newlen ; i < sa->count ; i++) {
+            free(sa->data[i]);
+            sa->data[i] = NULL;
+        }
     }
     sa->count = newlen;
 }
@@ -282,38 +291,44 @@ EXPORTED void strarray_truncate(strarray_t *sa, int newlen)
 EXPORTED const char *strarray_nth(const strarray_t *sa, int idx)
 {
     if ((idx = adjust_index_ro(sa, idx)) < 0)
-	return NULL;
+        return NULL;
     return sa->data[idx];
 }
 
+EXPORTED const char *strarray_safenth(const strarray_t *sa, int idx)
+{
+    const char *v = strarray_nth(sa, idx);
+    return v ? v : "";
+}
+
 EXPORTED char *strarray_join(const strarray_t *sa, const char *sep)
 {
     int seplen = (sep ? strlen(sep) : 0);
     int len = 0;
     int i;  /* array index */
     int j;  /* index into non-sparse logical subset of the array
-	     * i.e. doesn't count NULLs */
+             * i.e. doesn't count NULLs */
     char *buf, *p;
 
     for (i = 0, j = 0 ; i < sa->count ; i++) {
-	if (sa->data[i])
-	    len += strlen(sa->data[i]) + (j++ ? seplen : 0);
+        if (sa->data[i])
+            len += strlen(sa->data[i]) + (j++ ? seplen : 0);
     }
 
     if (!len)
-	return NULL;
-    len++;	/* room for NUL terminator */
+        return NULL;
+    len++;      /* room for NUL terminator */
     p = buf = xmalloc(len);
 
     for (i = 0, j = 0 ; i < sa->count ; i++) {
-	if (sa->data[i]) {
-	    if (j++ && sep) {
-		strcpy(p, sep);
-		p += strlen(p);
-	    }
-	    strcpy(p, sa->data[i]);
-	    p += strlen(p);
-	}
+        if (sa->data[i]) {
+            if (j++ && sep) {
+                strcpy(p, sep);
+                p += strlen(p);
+            }
+            strcpy(p, sa->data[i]);
+            p += strlen(p);
+        }
     }
 
     return buf;
@@ -327,16 +342,16 @@ EXPORTED strarray_t *strarray_splitm(char *buf, const char *sep, int flags)
     if (!buf) return sa;
 
     if (!sep)
-	sep = " \t\r\n";
+        sep = " \t\r\n";
 
     for (p = strtok(buf, sep) ; p ; p = strtok(NULL, sep)) {
-	if (flags & STRARRAY_TRIM) {
-	    while (Uisspace(*p)) p++;
-	    q = p + strlen(p);
-	    while (q > p && Uisspace(q[-1])) q--;
-	    *q = '\0';
-	}
-	if (*p) strarray_append(sa, p);
+        if (flags & STRARRAY_TRIM) {
+            while (Uisspace(*p)) p++;
+            q = p + strlen(p);
+            while (q > p && Uisspace(q[-1])) q--;
+            *q = '\0';
+        }
+        if (*p) strarray_append(sa, p);
     }
 
     free(buf);
@@ -346,14 +361,14 @@ EXPORTED strarray_t *strarray_splitm(char *buf, const char *sep, int flags)
 EXPORTED strarray_t *strarray_split(const char *line, const char *sep, int flags)
 {
     if (!line)
-	return strarray_new();
+        return strarray_new();
     return strarray_splitm(xstrdup(line), sep, flags);
 }
 
 EXPORTED strarray_t *strarray_nsplit(const char *buf, size_t len, const char *sep, int flags)
 {
     if (!len)
-	return strarray_new();
+        return strarray_new();
     return strarray_splitm(xstrndup(buf, len), sep, flags);
 }
 
@@ -366,9 +381,9 @@ EXPORTED char **strarray_takevf(strarray_t *sa)
     return d;
 }
 
-EXPORTED void strarray_sort(strarray_t *sa, compar_t *cmp)
+EXPORTED void strarray_sort(strarray_t *sa, strarray_cmp_fn_t *cmp)
 {
-    qsort(sa->data, sa->count, sizeof(char *), (__compar_fn_t)cmp);
+    qsort(sa->data, sa->count, sizeof(char *), cmp);
 }
 
 
@@ -377,20 +392,20 @@ EXPORTED void strarray_uniq(strarray_t *sa)
     int i;
 
     for (i = 1; i < sa->count; i++) {
-	if (!strcmpsafe(sa->data[i-1], sa->data[i]))
-	    free(strarray_remove(sa, i--));
+        if (!strcmpsafe(sa->data[i-1], sa->data[i]))
+            free(strarray_remove(sa, i--));
     }
 }
 
 /* common generic routine for the _find family */
 static int strarray_findg(const strarray_t *sa, const char *match, int starting,
-			  int (*compare)(const char *, const char *))
+                          int (*compare)(const char *, const char *))
 {
     int i;
 
     for (i = starting ; i < sa->count ; i++)
-	if (!compare(match, sa->data[i]))
-	    return i;
+        if (!compare(match, sa->data[i]))
+            return i;
     return -1;
 }
 
@@ -408,3 +423,20 @@ EXPORTED int strarray_size(const strarray_t *sa)
 {
     return sa->count;
 }
+
+EXPORTED int strarray_cmp(const strarray_t *a, const strarray_t *b)
+{
+    int as = strarray_size(a);
+    int bs = strarray_size(b);
+    int i;
+
+    /* test size first */
+    if (as != bs) return as - bs;
+
+    for (i = 0; i < as; i++) {
+        int res = strcmpsafe(strarray_nth(a, i), strarray_nth(b, i));
+        if (res) return res;
+    }
+
+    return 0;
+}
