@@ -206,6 +206,7 @@ enum {
 extern int set_caps(int stage, int is_master);
 extern int become_cyrus(int is_master);
 extern const char *cyrus_user(void);
+extern const char *cyrus_group(void);
 
 /* Some systems have very inefficient implementations of isdigit,
  * and we use it in a lot of inner loops
@@ -369,4 +370,14 @@ const char *makeuuid();
 void tcp_enable_keepalive(int fd);
 void tcp_disable_nagle(int fd);
 
+/*
+ * GCC_VERSION macro usage:
+ * #if GCC_VERSION > 60909    //GCC version 7 and above
+ *   do_something();
+ * #endif
+ */
+#define GCC_VERSION (__GNUC__ * 10000           \
+                     + __GNUC_MINOR__ * 100     \
+                     + __GNUC_PATCHLEVEL__)
+
 #endif /* INCLUDED_UTIL_H */
