@@ -46,8 +46,8 @@
 #include <config.h>
 
 char* notify_log(const char *class, const char *priority,
-		 const char *user, const char *mailbox,
-		 int nopt, char **options,
-		 const char *message);
+                 const char *user, const char *mailbox,
+                 int nopt, char **options,
+                 const char *message, const char *fname);
 
 #endif /* _NOTIFY_LOG_H_ */
