@@ -37,8 +37,6 @@
  * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * $Id: codes.h,v 1.6 2010/01/06 17:01:55 murch Exp $
  */
 
 #ifndef _CODES_H_
@@ -52,8 +50,4 @@
 #define SIEVE_BADPORT -5
 #define SIEVE_NOEXIST -6
 
-
-#define TRUE 1
-#define FALSE 0
-
 #endif
