@@ -43,6 +43,7 @@
 #ifndef SIEVE_BYTECODE_H
 #define SIEVE_BYTECODE_H
 
+#include <stddef.h>
 
 /* for debugging*/
 #define DUMPCODE 0
@@ -54,7 +55,7 @@
 
 
 /* yes, lots of these are superfluous, it's for clarity */
-typedef union 
+typedef union
 {
     int op; /* OPTYPE */
     int value;
@@ -78,7 +79,7 @@ struct bytecode_info
 /* For sanity during input on 64-bit platforms.
  * str should only be accessed as (char *)&str, but given the use of
  * unwrap_string, this should be OK */
-typedef union 
+typedef union
 {
     int op; /* OPTYPE */
     int value;
@@ -105,8 +106,10 @@ typedef union
  * version 0x07 scripts implemented updated INCLUDE (:once and :optional)
  * version 0x08 scripts implemented DATE and INDEX extensions
  * version 0x09 scripts implemented IMAP4FLAGS extension
+ * version 0x10 scripts implemented Mailbox and Metadata (RFC5490)
+ * version 0x11 scripts implemented Variables extension (RFC5229)
  */
-#define BYTECODE_VERSION 0x09
+#define BYTECODE_VERSION 0x11
 #define BYTECODE_MIN_VERSION 0x03 /* minimum supported version */
 #define BYTECODE_MAGIC "CyrSBytecode"
 #define BYTECODE_MAGIC_LEN 12 /* Should be multiple of 4 */
@@ -118,38 +121,45 @@ typedef union
 enum bytecode {
     B_STOP,
 
-    B_KEEP_ORIG,	/* legacy keep w/o support for :copy and :flags */
+    B_KEEP_ORIG,        /* legacy keep w/o support for :copy and :flags */
     B_DISCARD,
-    B_REJECT,		/* require reject */
-    B_FILEINTO_ORIG,	/* legacy fileinto w/o support for :copy */
-    B_REDIRECT_ORIG,	/* legacy redirect w/o support for :copy */
+    B_REJECT,           /* require reject */
+    B_FILEINTO_ORIG,    /* legacy fileinto w/o support for :copy */
+    B_REDIRECT_ORIG,    /* legacy redirect w/o support for :copy */
 
     B_IF,
-  
-    B_MARK,		/* require imapflags */
-    B_UNMARK,		/* require imapflags */
 
-    B_ADDFLAG,		/* require imap4flags */
-    B_SETFLAG,		/* require imap4flags */
-    B_REMOVEFLAG,	/* require imap4flags */
+    B_MARK,             /* require imapflags */
+    B_UNMARK,           /* require imapflags */
 
-    B_NOTIFY,		/* require notify */
-    B_DENOTIFY,		/* require notify */
+    B_ADDFLAG_ORIG,	/* legacy addflag w/o support for variables */
+    B_SETFLAG_ORIG,	/* legacy setflag w/o support for variables */
+    B_REMOVEFLAG_ORIG,	/* legacy removeflag w/o support for variables */
 
-    B_VACATION_ORIG,	/* legacy vacation w/o support for :seconds */
+    B_NOTIFY,           /* require notify */
+    B_DENOTIFY,         /* require notify */
+
+    B_VACATION_ORIG,    /* legacy vacation w/o support for :seconds */
     B_NULL,
     B_JUMP,
 
-    B_INCLUDE,		/* require include */
-    B_RETURN,		/* require include */
+    B_INCLUDE,          /* require include */
+    B_RETURN,           /* require include */
 
-    B_FILEINTO_COPY,	/* legacy fileinto w/o support for :flags */
+    B_FILEINTO_COPY,    /* legacy fileinto w/o support for :flags */
     B_REDIRECT,
 
-    B_VACATION,		/* require vacation */
+    B_VACATION,         /* require vacation */
 
     B_KEEP,
-    B_FILEINTO		/* require fileinto */
+    B_FILEINTO_FLAGS,   /* legacy fileinto w/o support for :create */
+    B_FILEINTO,         /* require mailbox, imap4flags, copy */
+
+    B_SET,		/* require variables */
+
+    B_ADDFLAG,		/* require imap4flags */
+    B_SETFLAG,		/* require imap4flags */
+    B_REMOVEFLAG	/* require imap4flags */
 };
 
 enum bytecode_comps {
@@ -161,87 +171,93 @@ enum bytecode_comps {
     BC_ANYOF,
     BC_ALLOF,
     BC_ADDRESS_PRE_INDEX,
-    BC_ENVELOPE,	/* require envelope */
+    BC_ENVELOPE,        /* require envelope */
     BC_HEADER_PRE_INDEX,
     BC_BODY,            /* require body */
     BC_DATE,            /* require date */
     BC_CURRENTDATE,     /* require date */
     BC_ADDRESS,
     BC_HEADER,
-    BC_HASFLAG		/* require imap4flags */
+    BC_HASFLAG,         /* require imap4flags */
+    BC_MAILBOXEXISTS,   /* require mailbox */
+    BC_METADATA,        /* require mboxmetadata */
+    BC_METADATAEXISTS,
+    BC_SERVERMETADATA,  /* require servermetadata */
+    BC_SERVERMETADATAEXISTS,
+    BC_STRING	        /* require variables */
 };
 
 /* currently one enum so as to help determine where values are being misused.
  * we have left placeholders incase we need to add more later to the middle */
 enum bytecode_tags {
-    /* Size Tests */
+    /* Size Tests (0-3) */
     B_OVER,
     B_UNDER,
 
     B_SIZE_PLACEHOLDER_1,
     B_SIZE_PLACEHOLDER_2,
-     
-    /* Relational Match Types */
-    B_GT,		/* require relational */
-    B_GE,		/* require relational */
-    B_LT,		/* require relational */
-    B_LE,		/* require relational */
-    B_EQ,		/* require relational */
-    B_NE,		/* require relational */
- 
+
+    /* Relational Match Types (4-11) */
+    B_GT,               /* require relational */
+    B_GE,               /* require relational */
+    B_LT,               /* require relational */
+    B_LE,               /* require relational */
+    B_EQ,               /* require relational */
+    B_NE,               /* require relational */
+
     B_RELATIONAL_PLACEHOLDER_1,
     B_RELATIONAL_PLACEHOLDER_2,
-   
-    /* Priorities */
-    B_LOW,		/* require notify */
-    B_NORMAL,		/* require notify */
-    B_HIGH,		/* require notify */
-    B_ANY,		/* require notify */
+
+    /* Priorities (12-19) */
+    B_LOW,              /* require notify */
+    B_NORMAL,           /* require notify */
+    B_HIGH,             /* require notify */
+    B_ANY,              /* require notify */
 
     B_PRIORITY_PLACEHOLDER_1,
     B_PRIORITY_PLACEHOLDER_2,
     B_PRIORITY_PLACEHOLDER_3,
     B_PRIORITY_PLACEHOLDER_4,
-    
-    /* Address Parts */
+
+    /* Address Parts (20-28) */
     B_ALL,
     B_LOCALPART,
     B_DOMAIN,
-    B_USER,		/* require subaddress */
-    B_DETAIL,		/* require subaddress */
-    
+    B_USER,             /* require subaddress */
+    B_DETAIL,           /* require subaddress */
+
     B_ADDRESS_PLACEHOLDER_1,
     B_ADDRESS_PLACEHOLDER_2,
     B_ADDRESS_PLACEHOLDER_3,
     B_ADDRESS_PLACEHOLDER_4,
 
-    /* Comparators */
+    /* Comparators (29-35) */
     B_ASCIICASEMAP,
     B_OCTET,
-    B_ASCIINUMERIC,	/* require comparator-i;ascii-numeric */
-    
+    B_ASCIINUMERIC,     /* require comparator-i;ascii-numeric */
+
     B_COMPARATOR_PLACEHOLDER_1,
     B_COMPARATOR_PLACEHOLDER_2,
     B_COMPARATOR_PLACEHOLDER_3,
     B_COMPARATOR_PLACEHOLDER_4,
- 
-    /* Match Types */
+
+    /* Match Types (36-45) */
     B_IS,
     B_CONTAINS,
     B_MATCHES,
-    B_REGEX,		/* require regex */
-    B_COUNT,		/* require relational */
-    B_VALUE,		/* require relational */
+    B_REGEX,            /* require regex */
+    B_COUNT,            /* require relational */
+    B_VALUE,            /* require relational */
 
     B_MATCH_PLACEHOLDER_1,
     B_MATCH_PLACEHOLDER_2,
     B_MATCH_PLACEHOLDER_3,
     B_MATCH_PLACEHOLDER_4,
 
-    /* Body Transforms */
-    B_RAW,		/* require body */
-    B_TEXT,		/* require body */
-    B_CONTENT,		/* require body */
+    /* Body Transforms (46-53) */
+    B_RAW,              /* require body */
+    B_TEXT,             /* require body */
+    B_CONTENT,          /* require body */
 
     B_TRANSFORM_PLACEHOLDER_1,
     B_TRANSFORM_PLACEHOLDER_2,
@@ -249,23 +265,23 @@ enum bytecode_tags {
     B_TRANSFORM_PLACEHOLDER_4,
     B_TRANSFORM_PLACEHOLDER_5,
 
-    /* Script locations */
-    B_PERSONAL,		/* require include */
-    B_GLOBAL,		/* require include */
+    /* Script locations (54-59) */
+    B_PERSONAL,         /* require include */
+    B_GLOBAL,           /* require include */
 
     B_LOCATION_PLACEHOLDER_1,
     B_LOCATION_PLACEHOLDER_2,
     B_LOCATION_PLACEHOLDER_3,
     B_LOCATION_PLACEHOLDER_4,
 
-    /* Zones */
+    /* Zones (60-63) */
     B_TIMEZONE,
     B_ORIGINALZONE,
 
     B_ZONE_PLACEHOLDER_1,
     B_ZONE_PLACEHOLDER_2,
 
-    /* Date Parts */
+    /* Date Parts (64-80) */
     B_YEAR,
     B_MONTH,
     B_DAY,
@@ -286,4 +302,18 @@ enum bytecode_tags {
     B_DATEPART_PLACEHOLDER_4
 };
 
+enum bytecode_variables_bitflags {
+    BFV_LOWER	        = 1<<0,
+    BFV_UPPER	        = 1<<1,
+    BFV_LOWERFIRST      = 1<<2,
+    BFV_UPPERFIRST      = 1<<3,
+    BFV_QUOTEWILDCARD   = 1<<4,
+    BFV_ENCODEURL	= 1<<5,
+    BFV_LENGTH		= 1<<6
+};
+
+enum bytecode_required_extensions {
+    BFE_VARIABLES       = 1<<0
+};
+
 #endif
